@@ -1,4 +1,6 @@
 """Contains functions for profilers."""
+from __future__ import annotations
+
 import collections
 import copy
 import datetime
@@ -8,16 +10,33 @@ import multiprocessing as mp
 import os
 import time
 import warnings
+from abc import abstractmethod
 from itertools import islice
+from multiprocessing.pool import Pool
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Iterator,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import numpy as np
 import psutil
 import scipy
+from pandas import DataFrame, Series
+from typing_extensions import Protocol
 
-from dataprofiler import settings
+from dataprofiler import profilers, settings
 
 
-def dict_merge(dct, merge_dct):
+def dict_merge(dct: Dict, merge_dct: Dict) -> None:
     # Recursive dictionary merge
     # Copyright (C) 2016 Paul Durivage <pauldurivage+github@gmail.com>
     #
@@ -45,7 +64,7 @@ def dict_merge(dct, merge_dct):
     :param merge_dct: dct merged into dct
     :return: None
     """
-    for k, v in merge_dct.items():
+    for k in merge_dct.keys():
         if (
             k in dct
             and isinstance(dct[k], dict)
@@ -65,12 +84,12 @@ class KeyDict(collections.defaultdict):
     https://www.drmaciver.com/2018/01/lazy-fisher-yates-shuffling-for-precise-rejection-sampling/
     """
 
-    def __missing__(self, key):
+    def __missing__(self, key: str) -> str:
         """Return key if key nonexistent."""
         return key
 
 
-def _combine_unique_sets(a, b):
+def _combine_unique_sets(a: List, b: List) -> List:
     """
     Unify two lists.
 
@@ -78,9 +97,9 @@ def _combine_unique_sets(a, b):
     :type b: list
     :rtype: list
     """
-    combined_list = None
+    combined_list: Set = set()
     if not a and not b:
-        combined_list = list()
+        combined_list = set()
     elif not a:
         combined_list = set(b)
     elif not b:
@@ -90,7 +109,9 @@ def _combine_unique_sets(a, b):
     return list(combined_list)
 
 
-def shuffle_in_chunks(data_length, chunk_size):
+def shuffle_in_chunks(
+    data_length: int, chunk_size: int
+) -> Generator[List[int], None, Any]:
     """
     Create shuffled indexes in chunks.
 
@@ -109,10 +130,10 @@ def shuffle_in_chunks(data_length, chunk_size):
     rng = np.random.default_rng(settings._seed)
 
     if "DATAPROFILER_SEED" in os.environ and settings._seed is None:
-        try:
-            seed_value = int(os.environ.get("DATAPROFILER_SEED"))
-            rng = np.random.default_rng(seed_value)
-        except ValueError:
+        seed = os.environ.get("DATAPROFILER_SEED")
+        if isinstance(seed, int):
+            rng = np.random.default_rng(int(seed))
+        else:
             warnings.warn("Seed should be an integer", RuntimeWarning)
 
     indices = KeyDict()
@@ -144,7 +165,7 @@ def shuffle_in_chunks(data_length, chunk_size):
         yield values
 
 
-def warn_on_profile(col_profile, e):
+def warn_on_profile(col_profile: str, e: Exception) -> None:
     """
     Return a warning if a given profile errors (tensorflow typically).
 
@@ -168,7 +189,7 @@ def warn_on_profile(col_profile, e):
     warnings.warn(warning_msg, RuntimeWarning, stacklevel=2)
 
 
-def partition(data, chunk_size):
+def partition(data: List, chunk_size: int) -> Generator[List, None, Any]:
     """
     Create a generator which returns data in specified chunk size.
 
@@ -181,7 +202,7 @@ def partition(data, chunk_size):
         yield data[idx : idx + chunk_size]
 
 
-def suggest_pool_size(data_size=None, cols=None):
+def suggest_pool_size(data_size: int = None, cols: int = None) -> Optional[int]:
     """
     Suggest the pool size based on resources.
 
@@ -189,7 +210,7 @@ def suggest_pool_size(data_size=None, cols=None):
     :type data_size: int
     :param cols: columns of the dataset
     :type cols: int
-    :return suggested_pool_size: suggeseted pool size
+    :return suggested_pool_size: suggested pool size
     :rtype suggested_pool_size: int
     """
     # Return if there's no data_size
@@ -218,7 +239,11 @@ def suggest_pool_size(data_size=None, cols=None):
     return suggested_pool_size
 
 
-def generate_pool(max_pool_size=None, data_size=None, cols=None):
+def generate_pool(
+    max_pool_size: int = None,
+    data_size: int = None,
+    cols: int = None,
+) -> Tuple[Optional[Pool], Optional[int]]:
     """
     Generate a multiprocessing pool to allocate functions too.
 
@@ -253,14 +278,16 @@ def generate_pool(max_pool_size=None, data_size=None, cols=None):
     return pool, max_pool_size
 
 
-def overlap(x1, x2, y1, y2):
+def overlap(
+    x1: Union[int, Any], x2: Union[int, Any], y1: Union[int, Any], y2: Union[int, Any]
+) -> bool:
     """Return True iff [x1:x2] overlaps with [y1:y2]."""
-    if not all([isinstance(i, int) for i in [x1, x2, y1, y2]]):
+    if not all(isinstance(i, int) for i in [x1, x2, y1, y2]):
         return False
     return (y1 <= x1 <= y2) or (y1 <= x2 <= y2) or (x1 <= y1 <= x2) or (x1 <= y2 <= x2)
 
 
-def add_nested_dictionaries(first_dict, second_dict):
+def add_nested_dictionaries(first_dict: Dict, second_dict: Dict) -> Dict:
     """
     Merge two dictionaries together and add values together.
 
@@ -270,7 +297,7 @@ def add_nested_dictionaries(first_dict, second_dict):
     :type second_dict: dict
     :return: merged dictionary
     """
-    merged_dict = {}
+    merged_dict: Dict = {}
     if isinstance(first_dict, collections.defaultdict):
         merged_dict = collections.defaultdict(first_dict.default_factory)
 
@@ -292,7 +319,7 @@ def add_nested_dictionaries(first_dict, second_dict):
     return merged_dict
 
 
-def biased_skew(df_series):
+def biased_skew(df_series: Series) -> float:
     """
     Calculate the biased estimator for skewness of the given data.
 
@@ -326,11 +353,11 @@ def biased_skew(df_series):
         return 0.0
 
     with np.errstate(all="ignore"):
-        skew = np.sqrt(n) * M3 / np.power(M2, 1.5)
+        skew: float = np.sqrt(n) * M3 / np.power(M2, 1.5)
     return skew
 
 
-def biased_kurt(df_series):
+def biased_kurt(df_series: Series) -> float:
     """
     Calculate the biased estimator for kurtosis of the given data.
 
@@ -364,11 +391,28 @@ def biased_kurt(df_series):
         return -3.0
 
     with np.errstate(all="ignore"):
-        kurt = n * M4 / np.power(M2, 2) - 3
+        kurt: float = n * M4 / np.power(M2, 2) - 3
     return kurt
 
 
-def find_diff_of_numbers(stat1, stat2):
+class Subtractable(Protocol):
+    """Protocol for annotating subtractable types."""
+
+    @abstractmethod
+    def __eq__(self: object, other: object) -> bool:
+        """Compare two objects."""
+        pass
+
+    @abstractmethod
+    def __sub__(self: T, other: T) -> Any:
+        """Compare two subtractables."""
+        pass
+
+
+T = TypeVar("T", bound=Subtractable)
+
+
+def find_diff_of_numbers(stat1: Optional[T], stat2: Optional[T]) -> Any:
     """
     Find the difference between two stats.
 
@@ -381,17 +425,18 @@ def find_diff_of_numbers(stat1, stat2):
     :type stat2: Union[int, float]
     :return: the difference of the stats
     """
-    diff = "unchanged"
     if stat1 is None and stat2 is None:
         pass
     elif stat1 is None or stat2 is None:
-        diff = [stat1, stat2]
+        return [stat1, stat2]
     elif stat1 != stat2:
-        diff = stat1 - stat2
-    return diff
+        return stat1 - stat2
+    return "unchanged"
 
 
-def find_diff_of_strings_and_bools(stat1, stat2):
+def find_diff_of_strings_and_bools(
+    stat1: Union[str, bool, None], stat2: Union[str, bool, None]
+) -> Union[List[Union[str, bool, None]], str]:
     """
     Find the difference between two stats.
 
@@ -404,13 +449,14 @@ def find_diff_of_strings_and_bools(stat1, stat2):
     :type stat2: Union[str, bool]
     :return: the difference of the stats
     """
-    diff = "unchanged"
     if stat1 != stat2:
-        diff = [stat1, stat2]
-    return diff
+        return [stat1, stat2]
+    return "unchanged"
 
 
-def find_diff_of_lists_and_sets(stat1, stat2):
+def find_diff_of_lists_and_sets(
+    stat1: Union[List, Set, None], stat2: Union[List, Set, None]
+) -> Union[List[Union[List, Set, None]], str]:
     """
     Find the difference between two stats.
 
@@ -424,11 +470,10 @@ def find_diff_of_lists_and_sets(stat1, stat2):
     :type stat2: Union[list, set]
     :return: the difference of the stats
     """
-    diff = "unchanged"
     if stat1 is None and stat2 is None:
         pass
     elif stat1 is None or stat2 is None:
-        diff = [stat1, stat2]
+        return [stat1, stat2]
     elif set(stat1) != set(stat2) or len(stat1) != len(stat2):
         temp_stat1 = list(copy.deepcopy(stat1))
         temp_stat2 = list(copy.deepcopy(stat2))
@@ -440,12 +485,14 @@ def find_diff_of_lists_and_sets(stat1, stat2):
         for element in shared:
             temp_stat1.remove(element)
 
-        diff = [temp_stat1, shared, temp_stat2]
+        return [temp_stat1, shared, temp_stat2]
 
-    return diff
+    return "unchanged"
 
 
-def find_diff_of_dates(stat1, stat2):
+def find_diff_of_dates(
+    stat1: Optional[datetime.datetime], stat2: Optional[datetime.datetime]
+) -> Union[List, str, None]:
     """
     Find the difference between two dates.
 
@@ -462,26 +509,26 @@ def find_diff_of_dates(stat1, stat2):
     :type stat1: datetime.datetime object
     :param stat2: the second statistical input
     :type stat2: datetime.datetime object
-    :return: Difference in stats
-    :rtype: str
+    :return: difference in stats
+    :rtype: Union[List, str]
     """
     # We can use find_diff_of_numbers since datetime objects
     # can be compared and subtracted naturally
-    diff = find_diff_of_numbers(stat1, stat2)
+    diff: Union[datetime.timedelta, List, str] = find_diff_of_numbers(stat1, stat2)
     if isinstance(diff, str):
         return diff
     if isinstance(diff, list):
-        # Deal with the case where one stat is None
         return [None if i is None else i.strftime("%x %X") for i in diff]
 
     # Must be timedelta object
     if diff.days >= 0:
         return "+" + str(diff)
-
     return "-" + str(abs(diff))
 
 
-def find_diff_of_dicts(dict1, dict2):
+def find_diff_of_dicts(
+    dict1: Optional[Dict], dict2: Optional[Dict]
+) -> Union[Dict, str]:
     """
     Find the difference between two dicts.
 
@@ -497,7 +544,10 @@ def find_diff_of_dicts(dict1, dict2):
     :return: Difference in the keys of each dict
     :rtype: dict
     """
-    diff = {}
+    dict1 = dict1 or {}
+    dict2 = dict2 or {}
+
+    diff: Dict = {}
     for key, value1 in dict1.items():
         value2 = dict2.get(key, None)
         if isinstance(value1, list):
@@ -518,12 +568,14 @@ def find_diff_of_dicts(dict1, dict2):
 
     # If both dicts have no keys, it is unchanged
     if diff == {}:
-        diff = "unchanged"
+        return "unchanged"
 
     return diff
 
 
-def find_diff_of_matrices(matrix1, matrix2):
+def find_diff_of_matrices(
+    matrix1: Optional[np.ndarray], matrix2: Optional[np.ndarray]
+) -> Optional[Union[np.ndarray, str]]:
     """
     Find the difference between two matrices.
 
@@ -534,21 +586,22 @@ def find_diff_of_matrices(matrix1, matrix2):
     :return: Difference in the matrix
     :rtype: list(list(float))
     """
-    diff = None
-
     if matrix1 is not None and matrix2 is not None:
-        mat1 = np.array(matrix1, dtype=np.float)
-        mat2 = np.array(matrix2, dtype=np.float)
+        mat1 = np.array(matrix1, dtype=np.float64)
+        mat2 = np.array(matrix2, dtype=np.float64)
 
         if mat1.shape == mat2.shape:
-            diff = mat1 - mat2
+            diff: np.ndarray = mat1 - mat2
             if ((diff == 0) | np.isnan(diff)).all():
-                diff = "unchanged"
+                return "unchanged"
+            return diff
 
-    return diff
+    return None
 
 
-def find_diff_of_dicts_with_diff_keys(dict1, dict2):
+def find_diff_of_dicts_with_diff_keys(
+    dict1: Optional[Dict], dict2: Optional[Dict]
+) -> Union[List[Dict], str]:
     """
     Find the difference between two dicts.
 
@@ -564,9 +617,12 @@ def find_diff_of_dicts_with_diff_keys(dict1, dict2):
     :return: Difference in the keys of each dict
     :rtype: list
     """
-    diff_1 = {}
-    diff_shared = {}
-    diff_2 = {}
+    dict1 = dict1 or {}
+    dict2 = dict2 or {}
+
+    diff_1: Dict = {}
+    diff_shared: Dict = {}
+    diff_2: Dict = {}
     for key, value1 in dict1.items():
         if key in dict2:
             value2 = dict2[key]
@@ -592,12 +648,12 @@ def find_diff_of_dicts_with_diff_keys(dict1, dict2):
 
     # If both dicts have no keys, it is unchanged
     if diff == [{}, {}, {}]:
-        diff = "unchanged"
+        return "unchanged"
 
     return diff
 
 
-def get_memory_size(data, unit="M"):
+def get_memory_size(data: Union[list, np.ndarray, DataFrame], unit: str = "M") -> float:
     """
     Get memory size of the input data.
 
@@ -613,14 +669,14 @@ def get_memory_size(data, unit="M"):
             "Currently only supports the "
             "memory size unit in {}".format(list(unit_map.keys()))
         )
-    memory_size = 0
+    memory_size: float = 0
     for sentence in data:
         memory_size += len(sentence.encode("utf-8"))
     memory_size /= 1024.0 ** unit_map[unit]  # Conversion based on unit_map
     return memory_size
 
 
-def method_timeit(method=None, name=None):
+def method_timeit(method: Callable = None, name: str = None) -> Callable:
     """
     Measure execution time of provided method.
 
@@ -632,9 +688,9 @@ def method_timeit(method=None, name=None):
     :type name: str
     """
 
-    def decorator(method, name_dec=None):
+    def decorator(method: Callable, name_dec: str = None) -> Callable:
         @functools.wraps(method)
-        def wrapper(self, *args, **kw):
+        def wrapper(self: Any, *args: Any, **kw: Any) -> Any:
             # necessary bc can't reassign external name
             name_dec = name
             if not name_dec:
@@ -653,8 +709,8 @@ def method_timeit(method=None, name=None):
 
 
 def perform_chi_squared_test_for_homogeneity(
-    categories1, sample_size1, categories2, sample_size2
-):
+    categories1: Dict, sample_size1: int, categories2: dict, sample_size2: int
+) -> Dict[str, Optional[Union[int, float]]]:
     """
     Perform a Chi Squared test for homogeneity between two groups.
 
@@ -669,7 +725,11 @@ def perform_chi_squared_test_for_homogeneity(
     :return: Results of the chi squared test
     :rtype: dict
     """
-    results = {"chi2-statistic": None, "df": None, "p-value": None}
+    results: Dict[str, Optional[Union[int, float]]] = {
+        "chi2-statistic": None,
+        "df": None,
+        "p-value": None,
+    }
 
     cat_counts = add_nested_dictionaries(categories1, categories2)
 
@@ -709,35 +769,38 @@ def perform_chi_squared_test_for_homogeneity(
     results["chi2-statistic"] = chi2_statistic
 
     # Calculate p-value, i.e. P(X > chi2_statistic)
-    p_value = 1 - scipy.stats.chi2(df).cdf(chi2_statistic)
+    p_value: float = 1 - scipy.stats.chi2(df).cdf(chi2_statistic)
     results["p-value"] = p_value
 
     return results
 
 
-def chunk(it, size):
+def chunk(lst: List, size: int) -> Iterator[Tuple]:
     """
     Chunk things out.
 
-    :param top_profile: Categories and respective counts of the second group
-    :type top_profile: list
-    :param other_profile: Number of samples in second group
-    :type other_profile: int
-    :return: Merge of two profile objects
-    :rtype: Profile
+    :param lst: List to chunk
+    :type lst: list
+    :param size: Size of each chunk
+    :type size: int
+    :return: Iterator that produces tuples of each chunk
+    :rtype: Iterator[Tuple]
     """
-    it = iter(it)
-    return iter(lambda: tuple(islice(it, size)), ())
+    iterator = iter(lst)
+    return iter(lambda: tuple(islice(iterator, size)), ())
 
 
-def merge(top_profile, other_profile=None):
+def merge(
+    top_profile: profilers.profile_builder.BaseProfiler,
+    other_profile: profilers.profile_builder.BaseProfiler = None,
+) -> profilers.profile_builder.BaseProfiler:
     """
     Merge two Profiles.
 
-    :param top_profile: Categories and respective counts of the second group
-    :type top_profile: list
-    :param other_profile: Number of samples in second group
-    :type other_profile: int
+    :param top_profile: First profile
+    :type top_profile: Profile
+    :param other_profile: Second profile
+    :type other_profile: Profile
     :return: Merge of two profile objects
     :rtype: Profile
     """
@@ -746,7 +809,10 @@ def merge(top_profile, other_profile=None):
     return top_profile + other_profile
 
 
-def merge_profile_list(list_of_profiles, pool_count=5):
+def merge_profile_list(
+    list_of_profiles: List[profilers.profile_builder.BaseProfiler],
+    pool_count: int = 5,
+) -> profilers.profile_builder.BaseProfiler:
     """Merge list of profiles into a single profile.
 
     :param list_of_profiles: Categories and respective counts of the second group
@@ -760,13 +826,13 @@ def merge_profile_list(list_of_profiles, pool_count=5):
     # remove the labeler model from the first profile object
     # assuming that the labeler models are all the same across each profile
     # in the list
-    for profile_idx, profile in enumerate(list_of_profiles):
-        data_labeler = list_of_profiles[profile_idx]._remove_data_labelers()
+    for profile in list_of_profiles:
+        data_labeler = profile._remove_data_labelers()
 
     while len(list_of_profiles) > 1:
-        list_of_profiles = chunk(list_of_profiles, 2)
+        list_of_profiles_iterator = chunk(list_of_profiles, 2)
         with mp.Pool(pool_count) as p:
-            list_of_profiles = p.starmap(merge, list_of_profiles)
+            list_of_profiles = p.starmap(merge, list_of_profiles_iterator)
 
     list_of_profiles[0]._restore_data_labelers(data_labeler)
     return list_of_profiles[0]
