@@ -68,6 +68,7 @@ class TestDataLabeler(unittest.TestCase):
     @staticmethod
     def _setup_mock_load_model(mock_load_model):
         model_mock = mock.Mock(spec=CharacterLevelCnnModel)
+        model_mock.__class__.__name__ = "CharacterLevelCnnModel"
         model_mock.set_num_labels = mock.Mock()
         mock_load_model.return_value = model_mock
         model_mock.requires_zero_mapping = True
@@ -128,7 +129,7 @@ class TestDataLabeler(unittest.TestCase):
             ".* != .*",
         ):
             mocked_model = mock.Mock(spec=CharacterLevelCnnModel)
-            mocked_model.__name__ = "FakeClassName"
+            mocked_model.__class__.__name__ = "FakeClassName"
             load_options = dict(model_class=mocked_model)
             UnstructuredDataLabeler._load_parameters("test/path", load_options)
 
@@ -147,7 +148,7 @@ class TestDataLabeler(unittest.TestCase):
             "preprocessor.\n .* != .*",
         ):
             mocked_preprocessor = mock.Mock(spec=data_processing.BaseDataPreprocessor)
-            mocked_preprocessor.__name__ = "FakeProcessorName"
+            mocked_preprocessor.__class__.__name__ = "FakeProcessorName"
             load_options = dict(preprocessor_class=mocked_preprocessor)
             UnstructuredDataLabeler._load_parameters("test/path", load_options)
 
