@@ -3,8 +3,9 @@ from __future__ import print_function
 import os
 import unittest
 
-from dataprofiler.profilers.column_profile_compilers import \
-    ColumnPrimitiveTypeProfileCompiler
+from dataprofiler.profilers.column_profile_compilers import (
+    ColumnPrimitiveTypeProfileCompiler,
+)
 
 from .test_base_column_profilers import AbstractTestColumnProfiler
 
@@ -14,7 +15,7 @@ test_root_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 class TestColumnDataTypeProfiler(AbstractTestColumnProfiler, unittest.TestCase):
 
     column_profiler = ColumnPrimitiveTypeProfileCompiler
-    profile_types = ['data_type', 'statistics', 'data_type_representation']
+    profile_types = ["data_type", "statistics", "data_type_representation"]
 
     def setUp(self):
         AbstractTestColumnProfiler.setUp(self)
@@ -24,5 +25,5 @@ class TestColumnDataTypeProfiler(AbstractTestColumnProfiler, unittest.TestCase):
         super(TestColumnDataTypeProfiler, cls).setUpClass()
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
